pub mod registry {
    tonic::include_proto!("bky.registry.api");
}

use futures::stream::Stream;
use std::time::Duration;
use tokio_stream::StreamExt;
use tonic::{server::ClientStreamingService, transport::Channel};

use registry::{metadata_ops_client::MetadataOpsClient, PullCreateReq};

fn generate_request() -> impl Stream<Item = PullCreateReq> {
    tokio_stream::iter(1..u8::MAX).map(|i| PullCreateReq {
        t_code: format!("t_code:{:02}", i),
        sendtype: i as i32,
    })
}

async fn create_by_pull(client: &mut MetadataOpsClient<Channel>, dur: Duration) {
    // let in_stream = generate_request().throttle(dur);
    let in_stream = generate_request();
    let respone = client.create_by_pull(in_stream).await.unwrap();
    let mut resp_stream = respone.into_inner();
    while let Some(received) = resp_stream.next().await {
        let received = received.unwrap();
        println!("received :{}", received.t_code);
    }

    let in_stream_2 = generate_request();
    let respone = client.create_by_pull(in_stream_2).await.unwrap();
    let mut resp_stream = respone.into_inner();
    while let Some(received) = resp_stream.next().await {
        let received = received.unwrap();
        println!("received :{}", received.t_code);
    }
}

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    let mut client = MetadataOpsClient::connect("http://192.168.110.183:50050")
        // let mut client = MetadataOpsClient::connect("http://192.168.110.183:50050")
        .await
        .unwrap();
    println!("start bidirectional stream echo ");
    create_by_pull(&mut client, Duration::from_micros(1)).await;
    Ok(())
}
